@@ -71,7 +71,7 @@ class _httprequesthandler(BaseHTTPServer.BaseHTTPRequestHandler):
     def do_write(self):
         try:
             self.do_hgweb()
-        except socket.error, inst:
+        except socket.error as inst:
             if inst[0] != errno.EPIPE:
                 raise
 
@@ -208,7 +208,7 @@ class _httprequesthandleropenssl(_httprequesthandler):
             import OpenSSL
             OpenSSL.SSL.Context
         except ImportError:
-            raise util.Abort(_("SSL support is unavailable"))
+            raise error.Abort(_("SSL support is unavailable"))
         ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_METHOD)
         ctx.use_privatekey_file(ssl_cert)
         ctx.use_certificate_file(ssl_cert)
@@ -226,7 +226,7 @@ class _httprequesthandleropenssl(_httprequesthandler):
         import OpenSSL
         try:
             _httprequesthandler.do_write(self)
-        except OpenSSL.SSL.SysCallError, inst:
+        except OpenSSL.SSL.SysCallError as inst:
             if inst.args[0] != errno.EPIPE:
                 raise
 
@@ -239,7 +239,7 @@ class _httprequesthandleropenssl(_httprequesthandler):
             pass
 
 class _httprequesthandlerssl(_httprequesthandler):
-    """HTTPS handler based on Pythons ssl module (introduced in 2.6)"""
+    """HTTPS handler based on Python's ssl module"""
 
     url_scheme = 'https'
 
@@ -249,7 +249,7 @@ class _httprequesthandlerssl(_httprequesthandler):
             import ssl
             ssl.wrap_socket
         except ImportError:
-            raise util.Abort(_("SSL support is unavailable"))
+            raise error.Abort(_("SSL support is unavailable"))
         httpserver.socket = ssl.wrap_socket(
             httpserver.socket, server_side=True,
             certfile=ssl_cert, ssl_version=ssl.PROTOCOL_TLSv1)
@@ -344,6 +344,6 @@ def create_server(ui, app):
     port = util.getport(ui.config('web', 'port', 8000))
     try:
         return cls(ui, app, (address, port), handler)
-    except socket.error, inst:
-        raise util.Abort(_("cannot start server at '%s:%d': %s")
+    except socket.error as inst:
+        raise error.Abort(_("cannot start server at '%s:%d': %s")
                          % (address, port, inst.args[1]))
