@@ -2,7 +2,7 @@
 # time it is invoked, and tests the encoding of those pathnames.
 #
 # It uses a simple probabilistic model to generate valid pathnames
-# that have proven likely to expose bugs and divergent behaviour in
+# that have proven likely to expose bugs and divergent behavior in
 # different encoding implementations.
 
 from mercurial import store
@@ -143,7 +143,6 @@ def genpath(rng, count):
 
     mink, maxk = 1, 4096
     def steps():
-        x, k = 0, mink
         for i in xrange(count):
             yield mink + int(round(math.sqrt((maxk - mink) * float(i) / count)))
     for k in steps():
